/**
  * A PDF viewer for C Suite.
  * Copyright 2019 CuboCore Group
  *
  * This file is a part of QtPDF module from Qt-Labs (https://code.qt.io/cgit/qt-labs/qtpdf.git/).
  * Suitable modifications have been made to meet the needs of CorePDF.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, vsit http://www.gnu.org/licenses/.
  **/

#ifndef QPDFVIEW_H
#define QPDFVIEW_H

#include "RenderOptions.hpp"
#include <QAbstractScrollArea>

namespace CorePDF {
class Document;
class Navigation;
}

class QPdfViewImpl;

class QPdfView : public QAbstractScrollArea
{
	Q_OBJECT;

	Q_PROPERTY(CorePDF::Document * document READ document WRITE setDocument NOTIFY documentChanged);

	Q_PROPERTY(PageLayout pageLayout READ pageLayout WRITE setPageLayout NOTIFY pageLayoutChanged);
	Q_PROPERTY(ZoomMode zoomMode READ zoomMode WRITE setZoomMode NOTIFY zoomModeChanged);
	Q_PROPERTY(qreal zoomFactor READ zoomFactor WRITE setZoomFactor NOTIFY zoomFactorChanged);

	Q_PROPERTY(int pageSpacing READ pageSpacing WRITE setPageSpacing NOTIFY pageSpacingChanged);
	Q_PROPERTY(QMargins documentMargins READ documentMargins WRITE setDocumentMargins NOTIFY documentMarginsChanged);

public:

	enum PageLayout
	{
		SinglePage,
		FacingPages,
		BookView,
		OverView
	};
	Q_ENUM(PageLayout)

	enum ZoomMode
	{
		FitToWidth,
		FitInView,
		CustomZoom
	};
	Q_ENUM(ZoomMode)

	explicit QPdfView(QWidget *parent = nullptr);
	~QPdfView();

	void setDocument(CorePDF::Document *document);
	CorePDF::Document *document() const;

	CorePDF::Navigation *pageNavigation() const;

	bool isLayoutContinuous() const;
	ZoomMode zoomMode() const;
	qreal zoomFactor() const;
	PageLayout pageLayout() const;

	int pageSpacing() const;
	void setPageSpacing(int spacing);

	QMargins documentMargins() const;
	void setDocumentMargins(QMargins margins);

	void searchText(const QString &str);

	CorePDF::RenderOptions::Rotation rotation() const;

	void setRotation(CorePDF::RenderOptions::Rotation);

public Q_SLOTS:
	void setLayoutContinuous(bool);
	void setZoomMode(ZoomMode mode);
	void setZoomFactor(qreal factor);
	void setPageLayout(PageLayout pgLyt);

Q_SIGNALS:
	void documentChanged(CorePDF::Document *document);
	void layoutContinuityChanged(bool continuous);
	void pageLayoutChanged(PageLayout pageLayout);
	void rotationChanged(CorePDF::RenderOptions::Rotation rotation);
	void zoomModeChanged(ZoomMode zoomMode);
	void zoomFactorChanged(qreal zoomFactor);
	void pageSpacingChanged(int pageSpacing);
	void documentMarginsChanged(QMargins documentMargins);

	void documentLoadingFailed();

	/** Search signals */
	void matchesFound(int numMatches);
	void searchComplete(int numMatches);

protected:
	void paintEvent(QPaintEvent *event) override;
	void resizeEvent(QResizeEvent *event) override;
	void scrollContentsBy(int dx, int dy) override;
	void keyPressEvent(QKeyEvent *kEvent) override;
	void wheelEvent(QWheelEvent *wEvent) override;

private:
	QPdfViewImpl *impl;
};

#endif // QPDFVIEW_H
