/**
  * A PDF viewer for C Suite.
  * Copyright 2019 CuboCore Group
  *
  * This file is a part of QtPDF module from Qt-Labs (https://code.qt.io/cgit/qt-labs/qtpdf.git/).
  * Suitable modifications have been made to meet the needs of CorePDF.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, vsit http://www.gnu.org/licenses/.
  **/

#ifndef QPDFVIEW_P_H
#define QPDFVIEW_P_H

#include <QPointer>
#include <QThread>
#include <QSize>
#include <QMargins>
#include <QRect>

#include "RenderOptions.hpp"
#include "qpdfview.h"

namespace CorePDF {
class Document;
class Renderer;
class RenderOptions;
class Navigation;
}

class QPdfView;

class PdfSearchThread : public QThread
{
	Q_OBJECT;

public:
	explicit PdfSearchThread(QObject *parent = nullptr);
	~PdfSearchThread();

	/** This will reset the search */
	void setDocument(CorePDF::Document *doc);

	/** Set the search string. Call searchPage(...) after this */
	void setSearchString(const QString &);

	/** Start or queue the search */
	void searchPage(int pageNo);

	/** Call searchPage(...) before calling results(...) */
	QVector<QRectF> results(int);

private:
	CorePDF::Document *mDoc;
	QString needle;

	QVector<int> pages;

	QHash<int, QVector<QRectF> > mResults;

	/** Stop completely */
	bool stop;

	/** Stop-other-loops-and-do-this_page flag */
	bool stop_others;

	/** Matches found so far */
	int matchCount;

protected:
	/** We perform the actual search here, and emit the signal */
	void run();

Q_SIGNALS:
	/** Results of @pageNo are ready */
	void resultsReady(int pageNo);

	void matchesFound(int);
	void searchComplete(int numMatches);

	/** Internal signal - used for prioritizing a particular page */
	void pendingRestart();
};

class QPdfViewImpl
{
public:
	explicit QPdfViewImpl(QPdfView *view);
	~QPdfViewImpl();

	void init();

	void documentStatusChanged();
	void currentPageChanged(int currentPage);
	void calculateViewport();
	void setViewport(QRect viewport);
	void updateScrollBars();

	void pageRendered(int pageNumber, QSize imageSize, const QImage& image, quint64 requestId);
	void invalidateDocumentLayout();

	qreal yPositionForPage(int page) const;

	qreal zoomFactor() const;
	void searchText(QString text, int pageNo);

	struct DocumentLayout
	{
		QSize             documentSize;
		QHash<int, QRect> pageGeometries;
	};

	DocumentLayout calculateDocumentLayout() const;
	DocumentLayout calculateDocumentLayoutSingle() const;
	DocumentLayout calculateDocumentLayoutFacing() const;
	DocumentLayout calculateDocumentLayoutBook() const;
	DocumentLayout calculateDocumentLayoutOverview() const;

	void updateDocumentLayout();

	void paintSearchRects(int, QImage&);

	QPointer<CorePDF::Document> m_document;
	CorePDF::Navigation *m_pageNavigation = nullptr;

	CorePDF::Renderer *m_pageRenderer = nullptr;
	CorePDF::RenderOptions m_renderOpts;

	bool m_continuous;
	QPdfView::PageLayout m_pageLayout;
	QPdfView::ZoomMode m_zoomMode;
	qreal m_zoomFactor;

	int m_pageSpacing;
	QMargins m_documentMargins;

	bool m_blockPageScrolling;

	QMetaObject::Connection m_documentStatusChangedConnection;

	QRect m_viewport;

	QVector<int> m_cachedPagesLRU;

	DocumentLayout m_documentLayout;

	qreal m_screenResolution; // pixels per point

	PdfSearchThread *m_searchThread = nullptr;
	QHash<int, QVector<QRectF> > searchRects;

private:
	QPdfView *view;
};

Q_DECLARE_TYPEINFO(QPdfViewImpl::DocumentLayout, Q_MOVABLE_TYPE);

#endif // QPDFVIEW_P_H
